import { tfaTypes, User } from 'entities';
import { BadRequestException } from '@nestjs/common';
import BaseAuthService from './base';
import { randomBytes } from 'crypto';
import * as notp from 'notp';
import { createTransport } from 'nodemailer';

const mailAccount = createTransport({
	host: 'mail.mattv.de',
	port: 587,
	secure: false,
	auth: {
		user: 'no-reply@mattv.de',
		pass: 'noreplyLONGPASS123'
	}
});

export default class TfaAuthService extends BaseAuthService {
	generateTfaSecret(): string {
		const set =
			'0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz!@#$%^&*()<>?/[]{},.:;';
		return randomBytes(32)
			.map((b) =>
				set.charCodeAt(Math.floor((b / 255.0) * (set.length - 1)))
			)
			.toString();
	}

	requiresTfa(user: User): boolean {
		return user.tfaType != tfaTypes.NONE;
	}

	async verifyTfa(
		user: User,
		token: string,
		type?: tfaTypes
	): Promise<boolean> {
		if (!type) type = user.tfaType;
		const delta = notp.totp.verify(token, user.tfaSecret, {
			window: 10
		});
		return (
			delta &&
			(type == tfaTypes.EMAIL ? delta.delta <= 0 : delta.delta == 0)
		);
	}

	async sendTfaMail(user: User) {
		await mailAccount.sendMail({
			from: 'fileserver@mattv.de',
			to: user.name,
			subject: 'Fileserver - EMail 2fa code',
			text: `Your code is: ${notp.totp.gen(
				user.tfaSecret
			)}\nIt is valid for 5 Minutes`
		});
	}

	async setupTfa(user: User): Promise<string> {
		if (user.tfaType != tfaTypes.NONE)
			throw new BadRequestException(
				'2 Factor authentication is already setup'
			);
		const secret = this.generateTfaSecret();
		user.tfaSecret = secret;
		await this.userRepo.save(user);
		return secret;
	}

	async setTfaType(user: User, type: tfaTypes) {
		user.tfaType = type;
		await this.userRepo.save(user);
	}
}
